/*
 * Completely Fair Scheduling (CFS) Class (SCHED_NORMAL/SCHED_BATCH)
 *
 *  Copyright (C) 2007 Red Hat, Inc., Ingo Molnar <mingo@redhat.com>
 *
 *  Interactivity improvements by Mike Galbraith
 *  (C) 2007 Mike Galbraith <efault@gmx.de>
 *
 *  Various enhancements by Dmitry Adamushko.
 *  (C) 2007 Dmitry Adamushko <dmitry.adamushko@gmail.com>
 *
 *  Group scheduling enhancements by Srivatsa Vaddagiri
 *  Copyright IBM Corporation, 2007
 *  Author: Srivatsa Vaddagiri <vatsa@linux.vnet.ibm.com>
 *
 *  Scaled math optimizations by Thomas Gleixner
 *  Copyright (C) 2007, Thomas Gleixner <tglx@linutronix.de>
 */

/*
 * Preemption granularity:
 * (default: 2 msec, units: nanoseconds)
 *
 * NOTE: this granularity value is not the same as the concept of
 * 'timeslice length' - timeslices in CFS will typically be somewhat
 * larger than this value. (to see the precise effective timeslice
 * length of your workload, run vmstat and monitor the context-switches
 * field)
 *
 * On SMP systems the value of this is multiplied by the log2 of the
 * number of CPUs. (i.e. factor 2x on 2-way systems, 3x on 4-way
 * systems, 4x on 8-way systems, 5x on 16-way systems, etc.)
 */
unsigned int sysctl_sched_granularity __read_mostly = 2000000000ULL/HZ;

/*
 * SCHED_BATCH wake-up granularity.
 * (default: 10 msec, units: nanoseconds)
 *
 * This option delays the preemption effects of decoupled workloads
 * and reduces their over-scheduling. Synchronous workloads will still
 * have immediate wakeup/sleep latencies.
 */
unsigned int sysctl_sched_batch_wakeup_granularity __read_mostly =
							10000000000ULL/HZ;

/*
 * SCHED_OTHER wake-up granularity.
 * (default: 1 msec, units: nanoseconds)
 *
 * This option delays the preemption effects of decoupled workloads
 * and reduces their over-scheduling. Synchronous workloads will still
 * have immediate wakeup/sleep latencies.
 */
unsigned int sysctl_sched_wakeup_granularity __read_mostly = 1000000000ULL/HZ;

unsigned int sysctl_sched_stat_granularity __read_mostly;

/*
 * Initialized in sched_init_granularity():
 */
unsigned int sysctl_sched_runtime_limit __read_mostly;

unsigned int sysctl_sched_features __read_mostly =
			0 | 2 | 4 | 8 | 0 | 0 | 0 | 0;

extern struct sched_class fair_sched_class;

/**************************************************************
 * CFS operations on generic schedulable entities:
 */

static inline struct rq *rq_of(struct cfs_rq *cfs_rq)
{
	return container_of(cfs_rq, struct rq, cfs);
}

static inline struct sched_entity *cfs_rq_curr(struct cfs_rq *cfs_rq)
{
	struct rq *rq = rq_of(cfs_rq);

	if (unlikely(rq->curr->sched_class != &fair_sched_class))
		return NULL;

	return &rq->curr->se;
}

#define entity_is_task(se)	1

static inline struct task_struct *task_of(struct sched_entity *se)
{
	return container_of(se, struct task_struct, se);
}


/**************************************************************
 * Scheduling class tree data structure manipulation methods:
 */

/*
 * Enqueue an entity into the rb-tree:
 */
static inline void
__enqueue_entity(struct cfs_rq *cfs_rq, struct sched_entity *se)
{
	struct rb_node **link = &cfs_rq->tasks_timeline.rb_node;
	struct rb_node *parent = NULL;
	struct sched_entity *entry;
	s64 key = se->fair_key;
	int leftmost = 1;

	/*
	 * Find the right place in the rbtree:
	 */
	while (*link) {
		parent = *link;
		entry = rb_entry(parent, struct sched_entity, run_node);
		/*
		 * We dont care about collisions. Nodes with
		 * the same key stay together.
		 */
		if ((s64)(key - entry->fair_key) < 0) {
			link = &parent->rb_left;
		} else {
			link = &parent->rb_right;
			leftmost = 0;
		}
	}

	/*
	 * Maintain a cache of leftmost tree entries (it is frequently
	 * used):
	 */
	if (leftmost)
		cfs_rq->rb_leftmost = &se->run_node;

	rb_link_node(&se->run_node, parent, link);
	rb_insert_color(&se->run_node, &cfs_rq->tasks_timeline);
	update_load_add(&cfs_rq->load, se->load.weight);
	cfs_rq->nr_running++;
	se->on_rq = 1;
}

static inline void
__dequeue_entity(struct cfs_rq *cfs_rq, struct sched_entity *se)
{
	if (cfs_rq->rb_leftmost == &se->run_node)
		cfs_rq->rb_leftmost = NULL;
	rb_erase(&se->run_node, &cfs_rq->tasks_timeline);
	update_load_sub(&cfs_rq->load, se->load.weight);
	cfs_rq->nr_running--;
	se->on_rq = 0;
}

static inline struct rb_node * first_fair(struct cfs_rq *cfs_rq)
{
	if (cfs_rq->rb_leftmost)
		return cfs_rq->rb_leftmost;
	/* Cache the value returned by rb_first() */
	cfs_rq->rb_leftmost = rb_first(&cfs_rq->tasks_timeline);
	return cfs_rq->rb_leftmost;
}

static struct sched_entity * __pick_next_entity(struct cfs_rq *cfs_rq)
{
	return rb_entry(first_fair(cfs_rq), struct sched_entity, run_node);
}

/**************************************************************
 * Scheduling class statistics methods:
 */

/*
 * We rescale the rescheduling granularity of tasks according to their
 * nice level, but only linearly, not exponentially:
 */
static s64
niced_granularity(struct sched_entity *curr, unsigned long granularity)
{
	u64 tmp;

	/*
	 * Negative nice levels get the same granularity as nice-0:
	 */
	if (curr->load.weight >= NICE_0_LOAD)
		return granularity;
	/*
	 * Positive nice level tasks get linearly finer
	 * granularity:
	 */
	tmp = curr->load.weight * (u64)granularity;
	return (s64) (tmp >> NICE_0_SHIFT);
}

static inline void
limit_wait_runtime(struct cfs_rq *cfs_rq, struct sched_entity *se)
{
	s64 limit = sysctl_sched_runtime_limit;

	/*
	 * Niced tasks have the same history dynamic range as
	 * non-niced tasks:
	 */
	if (unlikely(se->wait_runtime > limit)) {
		se->wait_runtime = limit;
		schedstat_inc(se, wait_runtime_overruns);
		schedstat_inc(cfs_rq, wait_runtime_overruns);
	}
	if (unlikely(se->wait_runtime < -limit)) {
		se->wait_runtime = -limit;
		schedstat_inc(se, wait_runtime_underruns);
		schedstat_inc(cfs_rq, wait_runtime_underruns);
	}
}

static inline void
__add_wait_runtime(struct cfs_rq *cfs_rq, struct sched_entity *se, s64 delta)
{
	se->wait_runtime += delta;
	schedstat_add(se, sum_wait_runtime, delta);
	limit_wait_runtime(cfs_rq, se);
}

static void
add_wait_runtime(struct cfs_rq *cfs_rq, struct sched_entity *se, s64 delta)
{
	schedstat_add(cfs_rq, wait_runtime, -se->wait_runtime);
	__add_wait_runtime(cfs_rq, se, delta);
	schedstat_add(cfs_rq, wait_runtime, se->wait_runtime);
}

/*
 * Update the current task's runtime statistics. Skip current tasks that
 * are not in our scheduling class.
 */
static inline void
__update_curr(struct cfs_rq *cfs_rq, struct sched_entity *curr, u64 now)
{
	struct load_weight *lw = &cfs_rq->load;
	u64 delta_exec, delta_fair, delta_mine;
	struct rq *rq = rq_of(cfs_rq);
	struct task_struct *curtask = rq->curr;
	unsigned long load = lw->weight;

	if (unlikely(!load))
		return;

	delta_exec = curr->delta_exec;
	if (unlikely(delta_exec > curr->exec_max))
		curr->exec_max = delta_exec;

	curr->sum_exec_runtime += delta_exec;
	cfs_rq->exec_clock += delta_exec;

	delta_fair = calc_delta_fair(delta_exec, lw);

	/*
	 * Task already marked for preemption, do not burden
	 * it with the cost of not having left the CPU yet:
	 */
	if (unlikely(sysctl_sched_features & 1))
		if (unlikely(test_tsk_thread_flag(curtask, TIF_NEED_RESCHED)))
			return;

	delta_mine = calc_delta_mine(delta_exec, curr->load.weight, lw);

	cfs_rq->fair_clock += delta_fair;
	/*
	 * We executed delta_exec amount of time on the CPU,
	 * but we were only entitled to delta_mine amount of
	 * time during that period (if nr_running == 1 then
	 * the two values are equal)
	 * [Note: delta_mine - delta_exec is negative]:
	 */
	add_wait_runtime(cfs_rq, curr, delta_mine - delta_exec);
}

static void update_curr(struct cfs_rq *cfs_rq, u64 now)
{
	struct sched_entity *curr = cfs_rq_curr(cfs_rq);
	unsigned long delta_exec;

	if (unlikely(!curr))
		return;

	/*
	 * Get the amount of time the current task was running
	 * since the last time we changed load:
	 */
	delta_exec = now - curr->exec_start;

	curr->delta_exec += delta_exec;

	if (curr->delta_exec > sysctl_sched_stat_granularity) {
		__update_curr(cfs_rq, curr, now);
		curr->delta_exec = 0;
	}
	curr->exec_start = now;
}

static inline void
update_stats_wait_start(struct cfs_rq *cfs_rq, struct sched_entity *se, u64 now)
{
	se->wait_start_fair = cfs_rq->fair_clock;
	se->wait_start = now;
}

static inline s64 weight_s64(s64 calc, unsigned long weight, int shift)
{
	if (calc < 0) {
		calc = - calc * weight;
		return - (calc >> shift);
	}
	return (calc * weight) >> shift;
}

/*
 * Task is being enqueued - update stats:
 */
static inline void
update_stats_enqueue(struct cfs_rq *cfs_rq, struct sched_entity *se, u64 now)
{
	s64 key;

	/*
	 * Are we enqueueing a waiting task? (for current tasks
	 * a dequeue/enqueue event is a NOP)
	 */
	if (se != cfs_rq_curr(cfs_rq))
		update_stats_wait_start(cfs_rq, se, now);
	/*
	 * Update the key:
	 */
	key = cfs_rq->fair_clock;

	/*
	 * Optimize the common nice 0 case:
	 */
	if (likely(se->load.weight == NICE_0_LOAD)) {
		key -= se->wait_runtime;
	} else {
		s64 tmp;

		if (se->wait_runtime < 0) {
			tmp = -se->wait_runtime;
			key += (tmp * se->load.inv_weight) >>
					(WMULT_SHIFT - NICE_0_SHIFT);
		} else {
			tmp = se->wait_runtime * se->load.weight;
			key -= tmp >> NICE_0_SHIFT;
		}
	}

	se->fair_key = key;
}

/*
 * Note: must be called with a freshly updated rq->fair_clock.
 */
static inline void
__update_stats_wait_end(struct cfs_rq *cfs_rq, struct sched_entity *se, u64 now)
{
	s64 delta_fair, delta_wait;

	delta_wait = now - se->wait_start;
	if (unlikely(delta_wait > se->wait_max))
		se->wait_max = delta_wait;

	delta_fair = se->delta_fair;

	if (unlikely(se->load.weight != NICE_0_LOAD))
		delta_fair = weight_s64(delta_fair, se->load.weight,
						NICE_0_SHIFT);

	add_wait_runtime(cfs_rq, se, delta_fair);
}

static void
update_stats_wait_end(struct cfs_rq *cfs_rq, struct sched_entity *se, u64 now)
{
	s64 delta_fair = cfs_rq->fair_clock - se->wait_start_fair;

	se->delta_fair += delta_fair;
	if (unlikely(se->delta_fair >= sysctl_sched_stat_granularity)) {
		__update_stats_wait_end(cfs_rq, se, now);
		se->delta_fair = 0;
	}

	se->wait_start_fair = 0;
	se->wait_start = 0;
}

static inline void
update_stats_dequeue(struct cfs_rq *cfs_rq, struct sched_entity *se, u64 now)
{
	update_curr(cfs_rq, now);
	/*
	 * Mark the end of the wait period if dequeueing a
	 * waiting task:
	 */
	if (se != cfs_rq_curr(cfs_rq))
		update_stats_wait_end(cfs_rq, se, now);
}

/*
 * We are picking a new current task - update its stats:
 */
static inline void
update_stats_curr_start(struct cfs_rq *cfs_rq, struct sched_entity *se, u64 now)
{
	/*
	 * We are starting a new run period:
	 */
	se->exec_start = now;
}

/*
 * We are descheduling a task - update its stats:
 */
static inline void
update_stats_curr_end(struct cfs_rq *cfs_rq, struct sched_entity *se, u64 now)
{
	se->exec_start = 0;
}

/*
 * A task gets added back to the runnable tasks and gets
 * a small credit for the CPU time it missed out on, so
 * fix up all other runnable task's wait_runtime so that
 * the sum stays constant (around 0).
 *
 * Instead of looping over all runnable tasks in an O(N)
 * manner we move the fair clock back by a proportional
 * amount of the new wait_runtime this task adds to the pool.
 */
static void distribute_fair_add(struct cfs_rq *cfs_rq, s64 delta)
{
	struct sched_entity *curr = cfs_rq_curr(cfs_rq);
	s64 delta_fair = 0;

	if (!(sysctl_sched_features & 2))
		return;

	if (cfs_rq->nr_running) {
		delta_fair = div64_s(delta, cfs_rq->nr_running);
		/*
		 * The currently running task's next wait_runtime value does
		 * not depend on the fair_clock, so fix it up explicitly:
		 */
		 if (curr)
			add_wait_runtime(cfs_rq, curr, -delta_fair);
	}
	cfs_rq->fair_clock -= delta_fair;
}

/**************************************************
 * Scheduling class queueing methods:
 */

static void
__enqueue_sleeper(struct cfs_rq *cfs_rq, struct sched_entity *se, u64 now)
{
	unsigned long load = cfs_rq->load.weight;
	s64 delta_fair, prev_runtime;

	delta_fair = cfs_rq->fair_clock - se->sleep_start_fair;

	/*
	 * Fix up delta_fair with the effect of us running
	 * during the whole sleep period:
	 */
	if (!(sysctl_sched_features & 32))
		delta_fair = div64_s(delta_fair * load, load + se->load.weight);

	delta_fair = weight_s64(delta_fair, se->load.weight, NICE_0_SHIFT);

	prev_runtime = se->wait_runtime;
	__add_wait_runtime(cfs_rq, se, delta_fair);
	delta_fair = se->wait_runtime - prev_runtime;

	/*
	 * We move the fair clock back by a load-proportional
	 * amount of the new wait_runtime this task adds to
	 * the 'pool':
	 */
	distribute_fair_add(cfs_rq, delta_fair);

	schedstat_add(cfs_rq, wait_runtime, se->wait_runtime);
}

static void
enqueue_sleeper(struct cfs_rq *cfs_rq, struct sched_entity *se, u64 now)
{
	struct task_struct *tsk = task_of(se);
	s64 delta_fair;

	if ((entity_is_task(se) && tsk->policy == SCHED_BATCH) ||
						 !(sysctl_sched_features & 4))
		return;

	delta_fair = cfs_rq->fair_clock - se->sleep_start_fair;
	se->delta_fair += delta_fair;
	if (unlikely(se->delta_fair >= sysctl_sched_stat_granularity)) {
		__enqueue_sleeper(cfs_rq, se, now);
		se->delta_fair = 0;
	}

	se->sleep_start_fair = 0;

#ifdef CONFIG_SCHEDSTATS
	if (se->sleep_start) {
		u64 delta = now - se->sleep_start;

		if ((s64)delta < 0)
			delta = 0;

		if (unlikely(delta > se->sleep_max))
			se->sleep_max = delta;

		se->sleep_start = 0;
		se->sum_sleep_runtime += delta;
	}
	if (se->block_start) {
		u64 delta = now - se->block_start;

		if ((s64)delta < 0)
			delta = 0;

		if (unlikely(delta > se->block_max))
			se->block_max = delta;

		se->block_start = 0;
		se->sum_sleep_runtime += delta;
	}
#endif
}

static void enqueue_entity(struct cfs_rq *cfs_rq, struct sched_entity *se,
							 int wakeup, u64 now)
{
	/*
	 * Update the fair clock.
	 */
	update_curr(cfs_rq, now);

	if (wakeup)
		enqueue_sleeper(cfs_rq, se, now);

	update_stats_enqueue(cfs_rq, se, now);
	__enqueue_entity(cfs_rq, se);
}

static void dequeue_entity(struct cfs_rq *cfs_rq, struct sched_entity *se,
							 int sleep, u64 now)
{
	update_stats_dequeue(cfs_rq, se, now);
	if (sleep) {
		se->sleep_start_fair = cfs_rq->fair_clock;
#ifdef CONFIG_SCHEDSTATS
		if (entity_is_task(se)) {
			struct task_struct *tsk = task_of(se);

			if (tsk->state & TASK_INTERRUPTIBLE)
				se->sleep_start = now;
			if (tsk->state & TASK_UNINTERRUPTIBLE)
				se->block_start = now;
		}
		cfs_rq->wait_runtime -= se->wait_runtime;
#endif
	}
	__dequeue_entity(cfs_rq, se);
}

/*
 * Preempt the current task with a newly woken task if needed:
 */
static inline void
__check_preempt_curr_fair(struct cfs_rq *cfs_rq, struct sched_entity *se,
			  struct sched_entity *curr, unsigned long granularity)
{
	s64 __delta = curr->fair_key - se->fair_key;

	/*
	 * Take scheduling granularity into account - do not
	 * preempt the current task unless the best task has
	 * a larger than sched_granularity fairness advantage:
	 */
	if (__delta > niced_granularity(curr, granularity))
		resched_task(rq_of(cfs_rq)->curr);
}

static struct sched_entity * pick_next_entity(struct cfs_rq *cfs_rq, u64 now)
{
	struct sched_entity *se = __pick_next_entity(cfs_rq);

	/*
	 * Any task has to be enqueued before it get to execute on
	 * a CPU. So account for the time it spent waiting on the
	 * runqueue. (note, here we rely on pick_next_task() having
	 * done a put_prev_task_fair() shortly before this, which
	 * updated rq->fair_clock - used by update_stats_wait_end())
	 */
	update_stats_wait_end(cfs_rq, se, now);
	update_stats_curr_start(cfs_rq, se, now);

	return se;
}

static void
put_prev_entity(struct cfs_rq *cfs_rq, struct sched_entity *prev, u64 now)
{
	int updated = 0;

	/*
	 * If the task is still waiting for the CPU (it just got
	 * preempted), update its position within the tree and
	 * start the wait period:
	 */
	if ((sysctl_sched_features & 16) && entity_is_task(prev))  {
		struct task_struct *prevtask = task_of(prev);

		if (prev->on_rq &&
			test_tsk_thread_flag(prevtask, TIF_NEED_RESCHED)) {

			dequeue_entity(cfs_rq, prev, 0, now);
			enqueue_entity(cfs_rq, prev, 0, now);
			updated = 1;
		}
	}

	/*
	 * If still on the runqueue then deactivate_task()
	 * was not called and update_curr() has to be done:
	 */
	if (prev->on_rq && !updated)
		update_curr(cfs_rq, now);

	update_stats_curr_end(cfs_rq, prev, now);

	if (prev->on_rq)
		update_stats_wait_start(cfs_rq, prev, now);
}

static void entity_tick(struct cfs_rq *cfs_rq, struct sched_entity *curr)
{
	struct sched_entity *next;
	struct rq *rq = rq_of(cfs_rq);
	u64 now = __rq_clock(rq);

	/*
	 * Dequeue and enqueue the task to update its
	 * position within the tree:
	 */
	dequeue_entity(cfs_rq, curr, 0, now);
	enqueue_entity(cfs_rq, curr, 0, now);

	/*
	 * Reschedule if another task tops the current one.
	 */
	next = __pick_next_entity(cfs_rq);
	if (next == curr)
		return;

	if (entity_is_task(curr)) {
		struct task_struct *curtask = task_of(curr),
				   *nexttask = task_of(next);

		if ((rt_prio(nexttask->prio) &&
				(nexttask->prio < curtask->prio))) {
			resched_task(curtask);
			return;
		}
	}
	__check_preempt_curr_fair(cfs_rq, next, curr, sysctl_sched_granularity);
}

/**************************************************
 * CFS operations on tasks:
 */

static inline struct cfs_rq *task_cfs_rq(struct task_struct *p)
{
	return &task_rq(p)->cfs;
}

/*
 * The enqueue_task method is called before nr_running is
 * increased. Here we update the fair scheduling stats and
 * then put the task into the rbtree:
 */
static void
enqueue_task_fair(struct rq *rq, struct task_struct *p, int wakeup, u64 now)
{
	struct cfs_rq *cfs_rq = task_cfs_rq(p);
	struct sched_entity *se = &p->se;

	enqueue_entity(cfs_rq, se, wakeup, now);
}

/*
 * The dequeue_task method is called before nr_running is
 * decreased. We remove the task from the rbtree and
 * update the fair scheduling stats:
 */
static void
dequeue_task_fair(struct rq *rq, struct task_struct *p, int sleep, u64 now)
{
	struct cfs_rq *cfs_rq = task_cfs_rq(p);
	struct sched_entity *se = &p->se;

	dequeue_entity(cfs_rq, se, sleep, now);
}

/*
 * sched_yield() support is very simple via the rbtree: we just
 * dequeue the task and move it after the next task, which
 * causes tasks to roundrobin.
 */
static void yield_task_fair(struct rq *rq, struct task_struct *p)
{
	struct cfs_rq *cfs_rq = task_cfs_rq(p);
	struct sched_entity *se = &p->se;
	struct rb_node *first, *curr, *next;

	curr = &se->run_node;
	next = rb_next(curr);
	first = rb_first(&cfs_rq->tasks_timeline);
	if ((first == curr) && next)
		cfs_rq->rb_leftmost = next;
	else
		cfs_rq->rb_leftmost = first;
}

/*
 * Preempt the current task with a newly woken task if needed:
 */
static void check_preempt_curr_fair(struct rq *rq, struct task_struct *p)
{
	struct task_struct *curr = rq->curr;
	struct cfs_rq *cfs_rq = task_cfs_rq(curr);
	unsigned long gran;

	if (unlikely(rt_prio(p->prio))) {
		if (sysctl_sched_features & 8) {
			if (rt_prio(p->prio))
				update_curr(cfs_rq, rq_clock(rq));
		}
		resched_task(curr);
		return;
	}

	gran = sysctl_sched_wakeup_granularity;
	/*
	 * Batch tasks prefer throughput over latency:
	 */
	if (unlikely(p->policy == SCHED_BATCH))
		gran = sysctl_sched_batch_wakeup_granularity;

	__check_preempt_curr_fair(cfs_rq, &p->se, &curr->se, gran);
}

static struct task_struct * pick_next_task_fair(struct rq *rq, u64 now)
{
	struct cfs_rq *cfs_rq = &rq->cfs;
	struct sched_entity *se;

	if (unlikely(!cfs_rq->nr_running))
		return NULL;

	se = pick_next_entity(cfs_rq, now);

	return task_of(se);
}

/*
 * Account for a descheduled task:
 */
static void put_prev_task_fair(struct rq *rq, struct task_struct *prev, u64 now)
{
	put_prev_entity(task_cfs_rq(prev), &prev->se, now);
}

/**************************************************
 * Fair scheduling class load-balancing methods:
 */

/*
 * Load-balancing iterator. Note: while the runqueue stays locked
 * during the whole iteration, the current task might be
 * dequeued so the iterator has to be dequeue-safe. Here we
 * achieve that by always pre-iterating before returning
 * the current task:
 */
static inline struct task_struct *
__load_balance_iterator(struct rq *rq, struct rb_node *curr)
{
	struct task_struct *p;

	if (!curr)
		return NULL;

	p = rb_entry(curr, struct task_struct, se.run_node);
	rq->cfs.rb_load_balance_curr = rb_next(curr);

	return p;
}

static struct task_struct * load_balance_start_fair(struct rq *rq)
{
	return __load_balance_iterator(rq, first_fair(&rq->cfs));
}

static struct task_struct * load_balance_next_fair(struct rq *rq)
{
	return __load_balance_iterator(rq, rq->cfs.rb_load_balance_curr);
}

/*
 * scheduler tick hitting a task of our scheduling class:
 */
static void task_tick_fair(struct rq *rq, struct task_struct *curr)
{
	entity_tick(task_cfs_rq(curr), &curr->se);
}

/*
 * Share the fairness runtime between parent and child, thus the
 * total amount of pressure for CPU stays equal - new tasks
 * get a chance to run but frequent forkers are not allowed to
 * monopolize the CPU. Note: the parent runqueue is locked,
 * the child is not running yet.
 */
static void task_new_fair(struct rq *rq, struct task_struct *p)
{
	struct cfs_rq *cfs_rq = task_cfs_rq(p);
	struct sched_entity *se = &p->se;
	u64 now = rq_clock(rq);

	sched_info_queued(p);

	update_stats_enqueue(cfs_rq, se, now);
	/*
	 * Child runs first: we let it run before the parent
	 * until it reschedules once. We set up the key so that
	 * it will preempt the parent:
	 */
	p->se.fair_key = current->se.fair_key - niced_granularity(&rq->curr->se,
						sysctl_sched_granularity) - 1;
	/*
	 * The first wait is dominated by the child-runs-first logic,
	 * so do not credit it with that waiting time yet:
	 */
	if (sysctl_sched_features & 256)
		p->se.wait_start_fair = 0;

	/*
	 * The statistical average of wait_runtime is about
	 * -granularity/2, so initialize the task with that:
	 */
	if (sysctl_sched_features & 128)
		p->se.wait_runtime = -(s64)(sysctl_sched_granularity / 2);

	__enqueue_entity(cfs_rq, se);
	inc_nr_running(p, rq, now);
}

/*
 * All the scheduling class methods:
 */
struct sched_class fair_sched_class __read_mostly = {
	.enqueue_task		= enqueue_task_fair,
	.dequeue_task		= dequeue_task_fair,
	.yield_task		= yield_task_fair,

	.check_preempt_curr	= check_preempt_curr_fair,

	.pick_next_task		= pick_next_task_fair,
	.put_prev_task		= put_prev_task_fair,

	.load_balance_start	= load_balance_start_fair,
	.load_balance_next	= load_balance_next_fair,
	.task_tick		= task_tick_fair,
	.task_new		= task_new_fair,
};
